<?php

/**
 * @file
 * Contains entity_api class files.
 */

/**
 * The class used for studyroom_reservation entities
 */
class StudyroomReservation extends Entity {
  /**
   * Default constructor.
   */
  public function __construct($values = array()) {
    parent::__construct($values, 'studyroom_reservation');
  }

  /**
   * Default defaultLabel.
   */
  protected function defaultLabel() {
    return 'Reservation ' . $this->reservation_id;
  }

  /**
   * Default defaultUri.
   */
  protected function defaultUri() {
    return array('path' => STUDYROOM_RESERVATION_PATH . $this->reservation_id);
  }

  /**
   * Function to determine if reservation is locked.
   */
  public function isLocked() {
    return isset($this->locked) ? $this->locked : 0;
  }

  /**
   * Determine if a conflict exists.
   */
  public function checkConflict() {
    $conflict = FALSE;
    $reservations = studyroom_reservation_per_space_per_day($this->space_id, $this->start_time->format('Y-m-d'), $this);
    $this->start_time = $this->start_time->format('U');
    $this->end_time = $this->end_time->format('U');

    foreach ($reservations as $current) {
      if ($current->end_time <= $this->start_time) {
        // This start time is after this reservation end time, so OK.
        continue;
      }
      if ($this->end_time > $current->start_time) {
        // End time is past start time of this reservation, so no good.
        $conflict = TRUE;
        $this->conflict_resv_ids[] = $current->reservation_id;
      }
    }

    return $conflict;
  }
}

/**
 * The class used for studyroom_reservation type entities
 */
class StudyroomReservationType extends Entity {
  public $type;
  public $label;
  public $locked;

  /**
   * Default constructor.
   */
  public function __construct($values = array()) {
    parent::__construct($values, 'studyroom_reservation_type');
  }

  /**
   * Override save function to add default values.
   */
  public function save() {
    if (!isset($this->min_duration)) {
      $this->min_duration = $this->defaultMinDuration();
    }
    if (!isset($this->max_duration)) {
      $this->max_duration = $this->defaultMaxDuration();
    }
    parent::save();
  }

  /**
   * Function to determine if reservation type is locked.
   */
  public function isLocked() {
    return isset($this->locked) ? $this->locked : 0;
  }

  /**
   * Return default value.
   */
  public function defaultMinDuration() {
    return '900';
  }

  /**
   * Return default value.
   */
  public function defaultMaxDuration() {
    return '14400';
  }

  /**
   * Build array of options for min_duration.
   *
   * @return array
   *   array of options formated for use in select form element
   */
  public function optionsMinDuartion() {
    return drupal_map_assoc(array(0, 900, 1800, 3600, 7200, 10800, 14400), 'format_interval');
  }

  /**
   * Build array of options for max_duration.
   *
   * @return array
   *   array of options formated for use in select form element
   */
  public function optionsMaxDuartion() {
    $values = array(
      0, 900, 1800, 3600, 7200, 10800, 14400, 21600, 32400, 43200, 64800, 86400);
    return drupal_map_assoc($values, 'format_interval');
  }
}


/**
 * The Controller for StudyroomReservation entities
 */
class StudyroomReservationController extends EntityAPIController {
  /**
   * Default constructor.
   */
  public function __construct($entity_type) {
    parent::__construct($entity_type);
  }

  /**
   * Create a studyroom_reservation.
   *
   * We first set up the values that are specific to our studyroom_reservation
   * schema but then also go through the EntityAPIController function.
   */
  public function create(array $values = array()) {
    // Add values that are specific to our StudyroomReservation.
    $values += array(
      'reservation_id' => '',
      'is_new' => TRUE,
      'title' => '',
      'created' => '',
      'changed' => '',
    );

    $reservation = parent::create($values);
    return $reservation;
  }

  /**
   * Delete override.
   *
   * Override delete to prevent deletion of locked types.
   */
  public function delete($ids) {
    $entities = $ids ? $this->load($ids) : FALSE;
    if (!$entities) {
      // Do nothing, in case invalid or no ids have been passed.
      return;
    }

    foreach ($entities as $id => $entity) {
      if ($entity->locked) {
        drupal_set_message(t('Locked reservation cannot be deleted.'));
      }
      else {
        parent::delete(array($id));

        // If not admin type, notify user of delete.
        if ($entity->type != 'admin') {
          /*
          $entity->uid
          notify_user_hook here? OR use entity_delete in studyroom_user module?
          */
          if (user_access('fake')) {
            drupal_set_message(t('@todo Notify user about delete!'), 'status', FALSE);
          }
        }
      }
    }
  }

  /**
   * Delete locked reservartions.
   */
  public function deleteLocked($ids) {
    parent::delete($ids);
  }

  /**
   * Load override.
   *
   * Allow theming of reservation label for availabilty module.
   */
  public function load($ids = array(), $conditions = array()) {
    $reservations = parent::load($ids, $conditions);
    foreach ($reservations as &$reservation) {
      $function = 'theme_studyroom_reservation_' . $reservation->type . '_label';
      if (function_exists($function)) {
        $reservation->label = $function();
      }
      else {
        $reservation->label = 'RESERVED';
      }
    }

    return $reservations;
  }
}


/**
 * The Controller for StudyroomReservation entities.
 */
class StudyroomReservationTypeController extends EntityAPIControllerExportable {
  /**
   * Default constructor.
   */
  public function __construct($entity_type) {
    parent::__construct($entity_type);
  }

  /**
   * Create a studyroom_reservation type.
   *
   * We first set up the values that are specific to our reservation type
   * schema but then also go through the EntityAPIController function.
   */
  public function create(array $values = array()) {
    // Add values that are specific to our StudyroomReservation.
    $values += array(
      'reservation_type_id' => '',
      'is_new' => TRUE,
      'locked' => 0,
      'public' => 1,
    );
    $studyroom_reservation_type = parent::create($values);
    return $studyroom_reservation_type;
  }


  /**
   * Save override.
   */
  public function save($entity) {
    parent::save($entity);

    // Create new instance of field.
    studyroom_reservation_create_field_instances($entity->type);

    // Clear types cache and force menu rebuild.
    studyroom_reservation_types_reset();
    variable_set('menu_rebuild_needed', TRUE);
  }

  /**
   * Delete override.
   *
   * Override delete to prevent deletion of locked types.
   */
  public function delete($ids) {
    $entities = $ids ? $this->load($ids) : FALSE;
    if (!$entities) {
      // Do nothing, in case invalid or no ids have been passed.
      return;
    }

    foreach ($entities as $entity) {
      if ($entity->locked) {
        drupal_set_message(t('Locked reservation types cannot be deleted.'));
      }
      else {
        parent::delete(array('0' => $entity->type));
      }
    }

    // Clear types cache and force menu rebuild.
    studyroom_reservation_types_reset();
    variable_set('menu_rebuild_needed', TRUE);
  }
}
