<?php

/**
 * @file
 * Studyroom space editing UI.
 */

/**
 * UI controller.
 */
class StudyroomSpaceUIController extends EntityDefaultUIController {
  /**
   * Overrides hook_menu() defaults. 
   *
   * Main reason for doing this is that parent class hook_menu() is
   * optimized for entity type administration.
   */
  public function hook_menu() {

    $items = array();
    $id_count = count(explode('/', STUDYROOM_SPACE_PATH)) - 1;
    $wildcard = isset($this->entityInfo['admin ui']['menu wildcard']) ? $this->entityInfo['admin ui']['menu wildcard'] : '%' . $this->entityType;

    $items[$this->path] = array(
      'title' => 'Space',
      'description' => 'Add, edit and update spaces.',
      'page callback' => 'system_admin_menu_block_page',
      'access arguments' => array('access administration pages'),
      'file path' => drupal_get_path('module', 'system'),
      'file' => 'system.admin.inc',
      'type' => MENU_LOCAL_TASK,
    );

    // Change the add page menu to multiple types of entities.
    $items[$this->path . '/add'] = array(
      'title' => 'Add a space',
      'description' => 'Add a new space',
      'page callback'  => 'studyroom_space_add_page',
      'access callback'  => 'studyroom_space_access',
      'access arguments' => array('create-spaces'),
      'type' => MENU_NORMAL_ITEM,
      'weight' => 20,
      'file' => $this->entityInfo['admin ui']['file'],
      'file path' => drupal_get_path('module', $this->entityInfo['module']),
    );

    // Add menu items to add each different type of entity.
    foreach (studyroom_space_locations() as $type) {
      $items[$this->path . '/add/' . $type->type] = array(
        'title' => 'Add space to the ' . $type->label . ' location',
        'page callback' => 'studyroom_space_form_wrapper',
        'page arguments' => array(studyroom_space_create(array('type' => $type->type))),
        'access callback' => 'studyroom_space_access',
        'access arguments' => array('create', $type->type),
        'file' => $this->entityInfo['admin ui']['file'],
        'file path' => drupal_get_path('module', $this->entityInfo['module']),
      );
    }

    $items[STUDYROOM_SPACE_PATH . $wildcard] = array(
      'title callback' => 'studyroom_space_title',
      'title arguments' => array($id_count),
      'page callback' => 'studyroom_space_page_view',
      'page arguments' => array($id_count, 2, 3),
      'access callback' => 'studyroom_space_access',
      'access arguments' => array('view', $id_count),
    );
    $items[STUDYROOM_SPACE_PATH . $wildcard . '/view'] = array(
      'title' => 'View',
      'type' => MENU_DEFAULT_LOCAL_TASK,
      'weight' => -10,
    );
    $items[STUDYROOM_SPACE_PATH . $wildcard . '/edit'] = array(
      'title' => 'Edit',
      'page callback' => 'studyroom_space_form_wrapper',
      'page arguments' => array($id_count),
      'access callback' => 'studyroom_space_access',
      'access arguments' => array('edit', $id_count),
      'type' => MENU_LOCAL_TASK,
      'context' => MENU_CONTEXT_PAGE | MENU_CONTEXT_INLINE,
      'file' => $this->entityInfo['admin ui']['file'],
      'file path' => drupal_get_path('module', $this->entityInfo['module']),
    );
    $items[STUDYROOM_SPACE_PATH . $wildcard . '/delete'] = array(
      'title' => 'Delete',
      'page callback' => 'studyroom_space_delete_form_wrapper',
      'page arguments' => array(1),
      'access callback' => 'studyroom_space_access',
      'access arguments' => array('delete', $id_count),
      'type' => MENU_LOCAL_TASK,
      'context' => MENU_CONTEXT_INLINE,
      'weight' => 10,
      'file' => $this->entityInfo['admin ui']['file'],
      'file path' => drupal_get_path('module', $this->entityInfo['module']),
    );
    $items[STUDYROOM_SPACE_PATH . $wildcard . '/json/%/%'] = array(
      'page callback' => 'studyroom_space_json',
      'page arguments' => array($id_count, 3, 4),
      'type' => MENU_CALLBACK,
      'access callback' => 'studyroom_space_access',
      'access arguments' => array('view', $id_count),
    );
    return $items;
  }

  /**
   * Function to display add space content.
   *     
   * Create the markup for the add Space Entities page within the class
   * so it can easily be extended/overriden.
   */
  public function addPage() {
    $item = menu_get_item();
    $content = system_admin_menu_block($item);

    if (count($content) == 1) {
      $item = array_shift($content);
      drupal_goto($item['href']);
    }

    if (count($content) > 1) {
      return theme('studyroom_space_choose_location', array('content' => $content));
    }

    if (user_access('administer studyroom_space locations')) {
      $output = '<p>' . t("Space's cannot be added because you have not created any locations yet. Go to the !location to add a new location.", array('!location' => l(t('location creation page'), STUDYROOM_SPACE_MENU_LOCATION . '/add'))) . '</p>';
    }
    else {
      $output = '<p>' . t('You do not have permission to create any spaces ....') . '</p>';
    }

    return $output;
  }

}

/**
 * Page to add Space Entities.
 *
 * @todo Pass this through a proper theme function
 */
function studyroom_space_add_page() {
  $controller = entity_ui_controller('studyroom_space');
  return $controller->addPage();
}

/**
 * Form callback wrapper: create or edit a space.
 *
 * @param object $space
 *   The space object being edited by this form.
 *
 * @see studyroom_space_edit_form()
 */
function studyroom_space_form_wrapper($space) {
  return drupal_get_form('studyroom_space_form', $space);
}

/**
 * Form callback wrapper: delete a space.
 *
 * @param object $space
 *   The space object being deleted by this form.
 *
 * @see studyroom_space_delete_form()
 */
function studyroom_space_delete_form_wrapper($space) {
  return drupal_get_form('studyroom_space_delete_form', $space);
}

/**
 * Form callback: create or edit a space.
 */
function studyroom_space_form($form, &$form_state, $studyroom_space) {
  if (!isset($form_state['studyroom_space'])) {
    $form_state['studyroom_space'] = $studyroom_space;
  }
  else {
    $studyroom_space = $form_state['studyroom_space'];
    // Need to set title, as when called from entity view, title wrong.
    if (isset($studyroom_space->name)) {
      drupal_set_title($studyroom_space->name);
    }
  }

  // Add the default field elements.
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Space name'),
    '#default_value' => isset($studyroom_space->name) ? $studyroom_space->name : '',
    '#maxlength' => 255,
    '#required' => TRUE,
    '#weight' => -50,
  );
  $form['description'] = array(
    '#type' => 'text_format',
    '#title' => t('Description'),
    '#default_value' => isset($studyroom_space->description['value']) ? $studyroom_space->description['value'] : '',
    '#format' => isset($studyroom_space->description['format']) ? $studyroom_space->description['format'] : filter_default_format(),
    '#weight' => 0,
  );
  $form['capacity'] = array(
    '#type' => 'textfield',
    '#title' => t('Capacity'),
    '#size' => 4,
    '#default_value' => isset($studyroom_space->capacity) ? $studyroom_space->capacity : '',
    '#description' => t('How many people can this room hold.'),
    '#required' => TRUE,
    '#element_validate' => array('element_validate_integer_positive'),
  );

  /*$location = studyroom_space_location_load($studyroom_space->type);*/
  /*$form_state['location'] = $location;*/

  $langcode = entity_language('studyroom_space', $studyroom_space);
  // Simply use default language.
  $form['language'] = array(
    '#type' => 'value',
    '#value' => isset($langcode) ? $langcode : LANGUAGE_NONE,
  );

  $form['actions'] = array(
    '#type' => 'container',
    '#attributes' => array('class' => array('form-actions')),
    '#weight' => 400,
  );

  $form['additional_settings'] = array(
    '#type' => 'vertical_tabs',
    '#weight' => 99,
  );

  // Node options for administrators.
  $form['options'] = array(
    '#type' => 'fieldset',
    '#title' => t('Publishing options'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#group' => 'additional_settings',
    '#attributes' => array(
      'class' => array('node-form-options'),
    ),
    '#attached' => array(
      'js' => array(drupal_get_path('module', 'node') . '/node.js'),
    ),
    '#weight' => 95,
  );
  $form['options']['status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Published'),
    '#default_value' => isset($studyroom_space->status) ? $studyroom_space->status : 1,
  );

  $studyroom = new Studyroom();
  $form['rules'] = array(
    '#type' => 'fieldset',
    '#title' => t('Override Rules'),
    '#tree' => FALSE,
    '#collapsible' => FALSE,
    '#collapsed' => FALSE,
    '#group' => 'additional_settings',
  );
  $form['rules']['min_duration'] = array(
    '#type' => 'select',
    '#title' => t('Set minimum duration'),
    '#default_value' => isset($studyroom_space->min_duration) ? $studyroom_space->min_duration : '',
    '#options' => $studyroom->optionsMinDuartion(),
    '#description' => t("Override the minimum duration for a reservation for this space.
      If no value is specified, the value of %value will be used.", array(
        '%value' => format_interval($studyroom->getMinDuration($studyroom_space)))),
  );
  $form['rules']['max_duration'] = array(
    '#type' => 'select',
    '#title' => t('Set maximum duration'),
    '#default_value' => isset($studyroom_space->max_duration) ? $studyroom_space->max_duration : '',
    '#options' => $studyroom->optionsMaxDuartion(),
    '#description' => t("Override the maximum duration for a reservation for this space.
      If no value is specified, the value of %value will be used.", array(
        '%value' => format_interval($studyroom->getMaxDuration($studyroom_space)))),
  );
  $form['rules']['max_days_advance'] = array(
    '#type' => 'select',
    '#title' => t('Set maximum days in advance'),
    '#default_value' => isset($studyroom_space->max_days_advance) ? $studyroom_space->max_days_advance : '',
    '#options' => $studyroom->optionsMaxDayAdvance(),
    '#description' => t("Override the maximum days in advance a reservation can be made for this space.
      If no value is specified, the value of %value will be used.", array(
        '%value' => studyroom_format_day($studyroom->getMaxDaysAdvance()))),
  );

  // We add the form's #submit array to this button along with the actual submit
  // handler to preserve any submit handlers added by a form callback_wrapper.
  $submit = array();
  if (!empty($form['#submit'])) {
    $submit += $form['#submit'];
  }

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save space'),
    '#submit' => $submit + array('studyroom_space_form_submit'),
  );

  if (!empty($studyroom_space->name) && studyroom_space_access('delete', $studyroom_space)) {
    $form['actions']['delete'] = array(
      '#title' => 'delete',
      '#type' => 'submit',
      '#value' => t('Delete space'),
      '#submit' => array('studyroom_space_form_submit_delete'),
      '#limit_validation_errors' => array(),
    );
  }

  if (isset($studyroom_space->space_id)) {
    $form['actions']['cancel'] = array(
      '#markup' => l(t('Cancel'), STUDYROOM_SPACE_PATH . $studyroom_space->space_id),
    );
  }

  // We append the validate handler to #validate in case a form callback_wrapper
  // is used to add validate handlers earlier.
  $form['#validate'][] = 'studyroom_space_form_validate';

  field_attach_form('studyroom_space', $studyroom_space, $form, $form_state, $langcode);
  return $form;
}


/**
 * Form API validate callback for the studyroom_space form.
 */
function studyroom_space_form_validate(&$form, &$form_state) {
  $values = $form_state['values'];
  $studyroom_space = $form_state['studyroom_space'];

  $studyroom = new Studyroom();
  $studyroom->validateDurations($form, $form_state, 'studyroom_space', $studyroom_space);

  if (empty($values['max_days_advance'])) {
    form_set_value($form['rules']['max_days_advance'], NULL, $form_state);
  }

  // Notify field widgets to validate their data.
  field_attach_form_validate('studyroom_space', $studyroom_space, $form, $form_state);
}


/**
 * Form API submit callback for the studyroom_space form.
 *
 * Save the space and go back to the list of spaces.
 *
 * @todo remove hard-coded link
 */
function studyroom_space_form_submit(&$form, &$form_state) {
  $space = entity_ui_controller('studyroom_space')->entityFormSubmitBuildEntity($form, $form_state);

  // Add in created and changed times.
  if ($space->is_new = isset($space->is_new) ? $space->is_new : 0) {
    $space->created = time();
  }
  $space->changed = time();

  $space->save();
  $form_state['redirect'] = STUDYROOM_SPACE_MENU_ADMIN_SPACE;
}

/**
 * Form API submit callback for the delete button.
 *
 * Unset destination to fix views destination issue.
 */
function studyroom_space_form_submit_delete(&$form, &$form_state) {
  if (isset($_GET['destination'])) {
    unset($_GET['destination']);
  }
  $form_state['redirect'] = STUDYROOM_SPACE_PATH . $form_state['studyroom_space']->space_id . '/delete';
}


/**
 * Form callback: confirmation form for deleting a space.
 *
 * @see confirm_form()
 */
function studyroom_space_delete_form($form, &$form_state, $space) {
  $form_state['studyroom_space'] = $space;
  $form['#submit'][] = 'studyroom_space_delete_form_submit';
  $form = confirm_form($form,
    t('Are you sure you want to delete space %name?', array('%name' => $space->name)),
    STUDYROOM_SPACE_MENU_ADMIN_SPACE,
    '<p>' . t('This action cannot be undone.') . '</p>',
    t('Delete'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Submit callback for studyroom_space_delete_form.
 */
function studyroom_space_delete_form_submit($form, &$form_state) {
  $space = $form_state['studyroom_space'];
  studyroom_space_delete($space);

  drupal_set_message(t('The space %name has been deleted.', array('%name' => $space->name)));
  watchdog('space', 'Deleted space %name.', array('%name' => $space->name));

  $form_state['redirect'] = STUDYROOM_SPACE_MENU_ADMIN_SPACE;
}
